@@ -4,7 +4,7 @@ import numpy
 import dufte
 
 
-def test_plot(filename=None):
+def test_plot(filename=None, light=True):
     plt.style.use(dufte.style)
 
     numpy.random.seed(0)
@@ -28,12 +28,22 @@ def test_plot(filename=None):
     # plt.title("title")
     dufte.legend()
 
+    if not light:
+        plt.gca().set_facecolor("#2d2d2d")
+        plt.gcf().patch.set_facecolor("#2d2d2d")
+
     if filename:
-        plt.savefig(filename, transparent=True, bbox_inches="tight")
+        # <https://github.com/matplotlib/matplotlib/issues/17321>
+        plt.savefig(
+            filename,
+            transparent=True,
+            bbox_inches="tight",
+            facecolor=plt.gcf().get_facecolor(),
+        )
     else:
         plt.show()
 
 
 if __name__ == "__main__":
-    # test_plot("ex1.svg")
-    test_plot()
+    test_plot("ex1-light.svg")
+    # test_plot("ex1-dark.svg", light=False)
